# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, unicode_literals
from argparse import ArgumentParser
import yaml
import sys
import logging
import importlib
import os
import json
import pkg_resources


logging.basicConfig(
    level='INFO',
    format='[%(asctime)s] {%(filename)s:%(lineno)d} %(levelname)s - %(message)s',
    datefmt='%y%m%d %H:%M:%S'
)


class DefaultJob(object):

    def __init__(self, path, params):
        os.chdir(path)
        config_path = os.path.join(path, 'config.yaml')
        sys.path.insert(0, path)

        self.job_path = path
        with open(config_path) as f:
            self.configs = yaml.load(f)

        self.params = {}
        if params != 'None':
            self.params = json.loads(params)
        self.configure()

    @staticmethod
    def _get_function(name):
        logging.info('Using function `{}`'.format(name))
        _module, _function = name.rsplit(':', 1)
        _module = importlib.import_module(_module)
        return getattr(_module, _function)

    def configure(self):
        include_paths = self.configs.get('include_paths')
        if not include_paths:
            include_paths = []
        elif isinstance(include_paths, str):
            include_paths = [include_paths]

        include_paths.append(self.job_path)
        log_level = self.configs.get('log-level', 'INFO').upper()
        config_path = os.path.join(self.job_path, 'config.yaml')

        self.logger = logging.getLogger(__file__)
        self.logger.setLevel(
            getattr(
                logging,
                log_level))
        self.logger.info(
            'Loading config from {}'.format(config_path))
        __builtins__.d = self.logger.debug
        __builtins__.i = self.logger.info
        __builtins__.w = self.logger.warning
        __builtins__.e = self.logger.error

        if include_paths:
            include_paths = list(
                map(lambda x: os.path.abspath(x), include_paths))
            logging.info(
                'Adding {} to PYTHONPATH'.format(','.join(include_paths)))
            include_paths.extend(sys.path)
            sys.path = include_paths

        self._main_function = DefaultJob._get_function(
            self.configs.get('entry'))

    def run(self):

        self._main_function(self)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('path')
    parser.add_argument('params')
    options = parser.parse_args()
    loader = DefaultJob(options.path, options.params)
    loader.run()



# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, unicode_literals
from qogir.core.commands import BaseCommand, CommandError
from qogir.utils.commands import run_bash_command

import os
import logging
import qogir
import yaml
import json
import sys


class JobRunner(object):

    def __init__(self, path, **kwargs):
        if not os.path.exists(os.path.join(path, '.qogir')):
            raise CommandError('Invalid Job Path')
        self.root_dir = qogir.__path__[0]
        self.path = os.path.abspath(path)
        self.env = os.path.join(self.path, '.venv')

        config_path = os.path.join(path, 'config.yaml')
        self.config = yaml.load(open(config_path))

    def configure(self):
        job_type = self.config['job-type']
        self.command_ = getattr(
            self, 'get_{}_command'.format(job_type))()

    def get_default_command(self):
        tpl_path = os.path.join(
            self.root_dir,
            'templates', 'default.submit.sh.tpl')
        runner_path = os.path.join(
            self.root_dir,
            'core', 'jobs', 'default.py')
        _command = open(tpl_path).read()
        _command = _command.format(
            job_path=self.path,
            env=self.env,
            runner_path=runner_path,
            params=self.params
        )
        return _command

    def get_pyspark_command(self):

        tpl_path = os.path.join(
            self.root_dir, 'templates', 'pyspark.submit.sh.tpl')
        runner_path = os.path.join(
            self.root_dir, 'core', 'jobs', 'pyspark_.py')
        spark_python_path = os.path.join(
            self.env, 'bin', 'python')
        submit_exec_path = os.path.join(
            self.env, 'bin', 'spark-submit')
        _command = open(tpl_path).read()
        _command = _command.format(
            job_path=self.path,
            env_path=self.env,
            runner_path=runner_path,
            submit_exec_path=submit_exec_path,
            spark_python_path=spark_python_path,
            params=self.params,
            **self.config['command-params'])
        return _command

    def _check_venv(self):
        _include_paths = self.config.get('include_paths')
        if not _include_paths:
            include_paths = []
        else:
            if isinstance(_include_paths, str):
                _include_paths = [_include_paths]
            include_paths = [repr(
                _path) for _path in _include_paths]
        tpl_path = os.path.join(
            self.root_dir,
            'templates', 'check_venv.sh.tpl')
        _command = open(tpl_path).read()
        _command = _command.format(
            job_path=self.path,
            python=self.config['python'],
            include_paths=' '.join(include_paths))
        if run_bash_command(_command) != 0:
            raise CommandError(
                'Command finished with a nonzero return code.')

    def run(self, *args, **kwargs):
        self.params = None
        if 'params' in kwargs:
            if isinstance(kwargs['params'], dict):
                self.params = json.dumps(kwargs['params'])
            else:
                self.params = kwargs['params']
        self.configure()
        self._check_venv()
        if run_bash_command(self.command_) != 0:
            raise CommandError(
                'Command finished with a nonzero return code.')



from __future__ import absolute_import, print_function, unicode_literals
import subprocess
import sys


def run_bash_command(command):
    proc = subprocess.Popen(
        command,
        executable='/bin/bash',
        shell=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT
    )

    for line in iter(proc.stdout.readline, b''):
        print(line.strip().decode())

    proc.stdout.close()
    proc.wait()

    return proc.returncode



# -*- coding: utf-8 -*-

from __future__ import print_function, absolute_import, unicode_literals
from pyspark import SparkContext, SparkConf
from pyspark.sql import SparkSession
from argparse import ArgumentParser
import yaml
import sys
import logging
import importlib
import os
import zipfile
import json
import re
import uuid


logging.basicConfig(
    level='INFO',
    format='[%(asctime)s] {%(filename)s:%(lineno)d} %(levelname)s - %(message)s',
    datefmt='%y%m%d %H:%M:%S'
)


class SparkJob(object):

    def __init__(self, path, params):
        os.chdir(path)
        config_path = os.path.join(path, 'config.yaml')

        sys.path.insert(0, path)
        self.job_path = path
        with open(config_path) as f:
            self.configs = yaml.load(f)

        self.params = {}
        if params != 'None':
            self.params = json.loads(params)
        self.configure()

    @staticmethod
    def _get_function(name):
        logging.info('Using function `{}`'.format(name))
        _module, _function = name.rsplit(':', 1)
        _module = importlib.import_module(_module)
        return getattr(_module, _function)

    def zipdir(self, path, ziph, excludes=r'(.venv/)'):
        # ziph is zipfile handle
        for root, dirs, files in os.walk(path):
            for file_ in files:
                _fname = os.path.join(root, file_)[len(path):]
                if excludes and re.findall(
                        excludes, _fname, re.IGNORECASE):
                    continue
                if file_.endswith('.py'):
                    ziph.write(
                        os.path.join(root, file_),
                        os.path.join(root, file_)[len(path):])

    def _configure_logger(self):
        log_level = self.configs.get('log-level', 'INFO').upper()
        config_path = os.path.join(self.job_path, 'config.yaml')

        self.logger = logging.getLogger()
        self.logger.setLevel(
            getattr(
                logging,
                log_level))
        self.logger.info(
            'Loading config from {}'.format(config_path))

        __builtins__.d = self.logger.debug
        __builtins__.i = self.logger.info
        __builtins__.w = self.logger.warning
        __builtins__.e = self.logger.error

    def _configure_spark(self):
        app_name = self.configs.get('app-name', 'app')
        log_level = self.configs.get('log-level', 'INFO').upper()

        conf = SparkConf().setAppName(app_name)
        self.sc = SparkContext(conf=conf)
        self.sc.setLogLevel(log_level)
        self.spark = SparkSession.builder.appName(
            'spark-session').getOrCreate()

    def _configure_include_paths(self):
        include_paths = self.configs.get('include_paths')
        if not include_paths:
            include_paths = []
        elif isinstance(include_paths, str):
            include_paths = [include_paths]

        include_paths.append(self.job_path)

        if include_paths:
            self.temp_files = []
            logging.info(
                'Adding {} to PYTHONPATH'.format(','.join(include_paths)))
            for ix, _path in enumerate(include_paths):
                _zip_filename = '.dep.{}.zip'.format(uuid.uuid4().hex[: 8])
                self.logger.info('Zip directory {}'.format(_path))
                zipf = zipfile.ZipFile(
                    _zip_filename, 'w', allowZip64=True)
                self.zipdir(_path, zipf)
                zipf.close()
                self.sc.addPyFile(
                    os.path.join(self.job_path, _zip_filename))
                self.temp_files.append(_zip_filename)
            include_paths.extend(sys.path)
            sys.path = include_paths

    def configure(self):

        self._configure_logger()
        self._configure_spark()
        self._configure_include_paths()

        self._main_function = SparkJob._get_function(
            self.configs.get('entry'))

    def run(self):

        self._main_function(self)

    def cleanup(self):
        if hasattr(self, 'temp_files'):
            for f in getattr(self, 'temp_files'):
                os.remove(f)


if __name__ == '__main__':
    parser = ArgumentParser()
    parser.add_argument('path')
    parser.add_argument('params')
    options = parser.parse_args()
    loader = SparkJob(options.path, options.params)
    loader.run()
    loader.cleanup()



from __future__ import absolute_import, print_function

from qogir.core.commands import BaseCommand, CommandError
import re
import os
import qogir
import shutil


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument(
            'name', metavar='name',
            help='Name of app',
        )

        parser.add_argument(
            '--type', metavar='type',
            default='default',
            choices=['default', 'pyspark'],
            help='Type of app'
        )


    def _is_valid_name(self, name):
        if not re.match(r'^[a-zA-Z]+[\w_]+$', name):
            raise CommandError('Invalid project name')

    def check(self):
        pass

    def execute(self, **options):
        name = options.get('name')
        _type = options.get('type')

        self._is_valid_name(name)
        _template_dir = os.path.join(
            qogir.__path__[0], 'templates', '{}_job_template'.format(_type))

        shutil.copytree(_template_dir, name)

        for dir_name, subdir_list, file_list in os.walk(name):
            for _file in file_list:
                if not _file.endswith('.tpl'):
                    continue
                shutil.move(
                    os.path.join(dir_name, _file),
                    os.path.join(dir_name, _file[:-4]),
                    )

        print('Successfully create project {}'.format(name))



from __future__ import absolute_import, print_function

from qogir.core.commands import BaseCommand, CommandError
from qogir.core.runner import JobRunner
from subprocess import Popen, PIPE
import re
import os
import qogir
import shutil
import sys
import logging
import yaml

logging.basicConfig(
    level=logging.INFO,
    format='[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] %(message)s',
    datefmt='%y%m%d %H:%M:%S'
)


class Command(BaseCommand):

    def add_arguments(self, parser):

        parser.add_argument(
            'path', metavar='Path',
            help='Job path',
        )

        parser.add_argument(
            '--params', '-P', metavar='params',
            default=None,
            help='Params of the job, must be json',
        )

    def check(self):
        pass

    @staticmethod
    def run_bash_command(command):
        proc = Popen(
            command,
            stdout=sys.stdout,
            stderr=sys.stderr,
            executable='/bin/bash',
            shell=True,
        )
        proc.communicate()
        if proc.returncode != 0:
            raise CommandError('Command finished with a nonzero return code.')

    def execute(self, **options):
        path = os.path.abspath(options['path'])
        params = options['params']
        runner = JobRunner(path)
        runner.run(params=params)




from setuptools import setup, find_packages
from codecs import open
from os import path
import qogir

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
# with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
#     long_description = f.read()

long_description = ''

setup(
    name='qogir',

    # Versions should comply with PEP440.  For a discussion on single-sourcing
    # the version across setup.py and the project code, see
    # https://packaging.python.org/en/latest/single_source_version.html
    version=qogir.VERSION,

    description='Qogir: Make task easier',
    long_description=long_description,

    # The project's main homepage.
    url='',

    # Author details
    author='Jerevia',
    author_email='trilliondawn@gmail.com',

    # Choose your license
    license='MIT',

    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',

        # Pick your license as you wish (should match "license" above)
        'License :: OSI Approved :: MIT License',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],

    # What does your project relate to?
    keywords='qogir spark',

    packages=find_packages(),

    include_package_data=True,

    install_requires=[
        'pyyaml'
    ],

    python_requires='>=2.6',

    extras_require={
        'dev': [],
        'test': [],
    },

    entry_points={
        'console_scripts': [
            'qogir = qogir.core:execute_from_command_line',
        ],
    },
)



from qogir.core.commands import BaseCommand, CommandError
from qogir.utils.commands import run_bash_command

import os
import logging
import qogir
import yaml
import aiofiles
import asyncio
import json
import sys


class aobject(object):
    """Inheriting this class allows you to define an async __init__.

    So you can create objects by doing something like `await MyClass(params)`
    """
    async def __new__(cls, *a, **kw):
        instance = super().__new__(cls)
        await instance.__init__(*a, **kw)
        return instance

    async def __init__(self):
        pass


class AsyncJobRunner(aobject):

    async def __init__(self, path, **kwargs):
        if not os.path.exists(os.path.join(path, '.qogir')):
            raise CommandError('Invalid Job Path')
        self.root_dir = qogir.__path__[0]
        self.path = os.path.abspath(path)
        self.env = os.path.join(self.path, '.venv')
        self.params = None
        config_path = os.path.join(path, 'config.yaml')
        async with aiofiles.open(config_path) as f:
            self.config = yaml.load(await f.read())

    async def configure(self):
        job_type = self.config['job-type']
        self.command_ = await getattr(
            self, 'get_{}_command'.format(job_type))()

    async def get_default_command(self):
        tpl_path = os.path.join(
            self.root_dir,
            'templates', 'default.submit.sh.tpl')
        runner_path = os.path.join(
            self.root_dir,
            'core', 'jobs', 'default.py')
        _command = open(tpl_path).read()
        _command = _command.format(
            job_path=self.path,
            env=self.env,
            runner_path=runner_path,
            params=self.params
        )
        return _command

    async def get_pyspark_command(self):

        tpl_path = os.path.join(
            self.root_dir, 'templates', 'pyspark.submit.sh.tpl')
        runner_path = os.path.join(
            self.root_dir, 'core', 'jobs', 'pyspark_.py')
        spark_python_path = os.path.join(
            self.env, 'bin', 'python')
        submit_exec_path = os.path.join(
            self.env, 'bin', 'spark-submit')
        _command = open(tpl_path).read()
        _command = _command.format(
            job_path=self.path,
            env_path=self.env,
            runner_path=runner_path,
            submit_exec_path=submit_exec_path,
            spark_python_path=spark_python_path,
            params=self.params,
            **self.config['command-params'])
        return _command

    async def _check_venv(self):
        _include_paths = self.config.get('include_paths')
        if not _include_paths:
            include_paths = []
        else:
            if isinstance(_include_paths, str):
                _include_paths = [_include_paths]
            include_paths = [repr(
                _path) for _path in _include_paths]
        tpl_path = os.path.join(
            self.root_dir,
            'templates', 'check_venv.sh.tpl')
        async with aiofiles.open(tpl_path) as f:
            _command = await f.read()
        _command = _command.format(
            job_path=self.path,
            python=self.config['python'],
            include_paths=' '.join(include_paths))
        if await self.run_bash_command(_command) != 0:
            raise CommandError(
                'Command finished with a nonzero return code.')

    async def run(self, *args, **kwargs):
        self.params = None
        if 'params' in kwargs:
            if isinstance(kwargs['params'], dict):
                self.params = json.dumps(kwargs['params'])
            else:
                self.params = kwargs['params']
        await self.configure()
        await self._check_venv()
        if await self.run_bash_command(self.command_) != 0:
            raise CommandError(
                'Command finished with a nonzero return code.')

    async def run_bash_command(self, cmd):
        asyncio.subprocess.PIPE = sys.stdout
        proc = await asyncio.create_subprocess_shell(
            cmd)
        await proc.communicate()
        return proc.returncode